from openfisca_us.model_api import *


class ny_federal_ctc(Variable):
    value_type = float
    entity = TaxUnit
    label = "NY federal CTC"
    unit = USD
    documentation = "The version of the federal ACTC used to determine the NY Empire State Child Credit."
    definition_period = YEAR
    defined_for = StateCode.NY

    formula = sum_of_variables(
        ["ny_federal_actc", "ny_federal_non_refundable_ctc"]
    )



from openfisca_us.model_api import *


class ma_part_a_taxable_capital_gains_income(Variable):
    value_type = float
    entity = TaxUnit
    label = "MA Part A taxable income from capital gains"
    unit = USD
    definition_period = YEAR
    reference = "https://www.mass.gov/info-details/mass-general-laws-c62-ss-3"
    defined_for = StateCode.MA

    def formula(tax_unit, period, parameters):
        part_a_taxable_income = tax_unit("ma_part_a_taxable_income", period)
        part_a_taxable_interest_dividends = tax_unit(
            "ma_part_a_taxable_interest_dividend_income", period
        )
        return part_a_taxable_income - part_a_taxable_interest_dividends



from openfisca_us.model_api import *


class spm_unit_capped_work_childcare_expenses(Variable):
    value_type = float
    entity = SPMUnit
    label = "SPM unit work and childcare expenses"
    definition_period = YEAR
    unit = USD



from openfisca_us.model_api import *


class ny_federal_non_refundable_ctc(Variable):
    value_type = float
    entity = TaxUnit
    label = "NY federal non-refundable CTC"
    unit = USD
    documentation = "The version of the federal non-refundable CTC used to determine the NY Empire State Child Credit."
    definition_period = YEAR
    defined_for = StateCode.NY

    def formula(tax_unit, period, parameters):
        max_amount = tax_unit("ny_federal_ctc_max", period)
        limiting_tax = tax_unit("ctc_limiting_tax_liability", period)
        return min_(max_amount, limiting_tax)



from openfisca_us.model_api import *
from openfisca_core.taxbenefitsystems import TaxBenefitSystem


class ctc_limiting_tax_liability(Variable):
    value_type = float
    entity = TaxUnit
    label = "CTC-limiting tax liability"
    unit = USD
    documentation = "The tax liability used to determine the maximum amount of the non-refundable CTC. Excludes SALT from all calculations (this is an inaccuracy required to avoid circular dependencies)."
    definition_period = YEAR

    def formula(tax_unit, period, parameters):
        non_refundable_credits = parameters(
            period
        ).gov.irs.credits.non_refundable
        total_credits = sum(
            [
                tax_unit(credit, period)
                for credit in non_refundable_credits
                if credit not in ("non_refundable_ctc",)
            ]
        )
        with BranchedSimulation(tax_unit) as simulation:
            simulation.tax_benefit_system.neutralize_variable(
                "state_income_tax"
            )
            simulation.get_holder(
                "state_income_tax"
            ).variable = simulation.tax_benefit_system.variables[
                "state_income_tax"
            ]  # This is needed because neutralize_variable is only designed to work *before* simulations have started.
            tax_liability = simulation.calculate(
                "income_tax_before_credits", period
            )
            simulation.get_holder(
                "state_income_tax"
            ).variable.is_neutralized = False

        return max_(0, tax_liability - total_credits)



from openfisca_us.model_api import *


class tax_liability_if_itemizing(Variable):
    value_type = float
    entity = TaxUnit
    label = "Tax liability if itemizing"
    unit = USD
    definition_period = YEAR

    def formula(tax_unit, period, parameters):
        with BranchedSimulation(tax_unit) as simulation_if_itemizing:
            simulation_if_itemizing.set_input(
                "tax_unit_itemizes",
                period,
                np.ones((tax_unit.count,), dtype=bool),
            )
            values = simulation_if_itemizing.calculate(
                "federal_state_income_tax", period
            )
        return values
